//#include <windows.h>

#include <gtk/gtk.h>
#include <mysql.h>

/* Biblioteka stdlib.h zawiera funkcje atoi i atof call. */

#include <stdlib.h>

#include "support.h"

#define	SERVER  "localhost"
#define	LOGIN   "kbi_user"
#define PASS    "dengissa"
#define DB      "kbi"

#define BASELINE_CHART    350   // Wysokosc pelnego slupka
                                // w pikselach
#define BASELINE_MONTHS   370   // Odleglosc od gory slupka do
                                // nazwy miesiaca (w pikselach)
#define SCALE_DENOMINATOR 1235  // Wspolczynnik podzialu
#define BAR_WIDTH         25    // szerokosc slupkow

GtkWidget *frm_barline;

MYSQL           *conx;
MYSQL_RES       *result_set;
MYSQL_ROW       row;
gchar           *sql;

gchar *month_names[12] = {"Sty", "Luty", "Mar", "Kwie", "Maj", "Cze",
                          "Lip", "Sie", "Wrz", "Paz", "Lis", "Gru"};

gint month_bar_amounts[12];
gint month_line_amounts[12];
gint month_indents[12];

void draw_bar(gint bar_left, gint bar_width, gint bar_height, gchar *bar_color);

GdkGC *get_gc(gchar *in_color)
{   /* Ta funkcja zwroci kontekst urzadzenia graficznego w okreslonym
     * kolorze
     */
   GdkGC    *gc;
   GdkColor my_color;

   gc = gdk_gc_new(GTK_WIDGET(lookup_widget(frm_barline,
           "drawingarea1"))->window);

   if (!gdk_color_parse(in_color, &my_color))
      {
         g_print("Nie mozna przetworzyc koloru %s.\n", in_color);
         gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
               "statusbar1")), 1, "Nie zinterpretowano koloru.\n");
         return 0L;
      }

   if (!gdk_colormap_alloc_color(gdk_colormap_get_system(),
        &my_color, FALSE, TRUE))
      {
         g_print("Nie zlokalizowano mapy kolorow dla %s.\n", in_color);
         gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
               "statusbar1")), 1, "Nie zlokalizowano mapy kolorow.\n");
         return 0L;
      }

   gdk_gc_set_foreground(gc, &my_color);

   return gc;

}

void draw_axis_and_scales()
{  /* Ta funckaj sluzy do narysowania osi wykresu
    */
   GdkGC    *gc;
   gint     i;
   gint     from_left = 90;

   gc = get_gc("Black");

   gdk_draw_string(
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
       gc, 5, 100, "Obrot (slupek)");

   gdk_draw_string(
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
       GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
       gc, 5, 130, "Prognoza (linia)");

   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           gc, from_left -1, 10, from_left -1, BASELINE_MONTHS - 20);

   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           gc, from_left, BASELINE_MONTHS - 20, 550, BASELINE_MONTHS - 20);


   /* Okreslenie odstepu pomiedzy poszczegolnymi miesiacami
    * (nazwami i slupkami)
    */

   for (i=0; i<=11; i++)
     {
         month_indents[i] = from_left;

         gdk_draw_string(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->style->font,
           gc, from_left, BASELINE_MONTHS, month_names[i]);

         from_left += 40;

     }

}

void draw_bar_chart()
{
   gint i;

   /* Najpierw nalezy okreslic skale. W naszym przypadku, najwieksza
    * wartoscia jest 400000.  W pionie mamy do dyspozycji 340 punktow
    * ( od 10 do 350). 10 reprezentuje wartosc 0, a 340 - 400000.
    *
    * Aby przyjac odpowiednia skale dla wykresu, zalozymy dodatkowy
    * margines - zalozmy wiec watrtosc maksymalna rowna 420000.
    * Dzielac 420000 przez 340 dostepnych punktow otrzymamy 1235 - jeden
    * punkt na wykresie oznacza 1235. Jesli teraz bedziemy dzielic wartosc
    * z zakresu (0, 400000) przez 1235, to nigdy nie otrzymamy 340
    * (nie wyjdziemy poza powierzchnie przeznaczona na wykres).
    * W programie bedziemy skalowac kazda wartosc z wyznaczonego zakresu
    * i przedstawiac ja w formie slupka.
    *
    * Moglibysmy kazac programowi najpierw pobrac maksymalna wartosc
    * z bazy danych, a nastepnie dynamicznie wyznaczyc podzial,
    * w zaleznosci od danych w danym roku. Tu skalujemy wykres
    * "na sztywno".
    *
    * To wlasnie w ten sposob wyznaczono wartosc SCALE_DENOMINATOR,
    * zdefiniowana w czesci naglowkowej pliku.
    */

   /* Make repeat calls to draw_bar() for each month. */

   for (i=0;  i<=11; i++)
      {
         draw_bar(month_indents[i], BAR_WIDTH,
                  month_bar_amounts[i]/SCALE_DENOMINATOR, "Red");

      }
}

void draw_bar(gint bar_left, gint bar_width, gint bar_height, gchar *bar_color)
{
   gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc(bar_color), TRUE, bar_left,
             BASELINE_CHART - bar_height,
             bar_width, bar_height);

}

void draw_line(gint from_left_1, gint from_top_1,
               gint from_left_2, gint from_top_2)

{
   gdk_draw_line(
           GTK_WIDGET(lookup_widget(frm_barline, "drawingarea1"))->window,
           get_gc("Blue"), from_left_1, from_top_1, from_left_2, from_top_2);


}


void draw_line_chart()
{
   gint i;
   gint from_left, from_top;

   /* Najpierw trzeba okreslic punkt startowy dla pierwszego odcinka.
    * Bedzie to srodek slupka dla stycznia (w poziomie). Punkt osi y
    * zalezy od zalozen zarzadu (czyli jest rowny wartosci
    *  month_line_amounts[0]).
    */

   from_left = month_indents[0] + BAR_WIDTH/2;
   from_top  = BASELINE_CHART - month_line_amounts[0]/SCALE_DENOMINATOR;

   for (i=1;  i<=11; i++)
      {


         draw_line(from_left, from_top,
                   month_indents[i] + BAR_WIDTH/2,
                   BASELINE_CHART - month_line_amounts[i]/SCALE_DENOMINATOR);

         gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc("RoyalBlue"),
             TRUE, from_left - 2, from_top - 2, 3, 3);

         from_left = month_indents[i] + BAR_WIDTH/2;
         from_top  = BASELINE_CHART - month_line_amounts[i]/SCALE_DENOMINATOR;

      }

   /* Jako ostatni jest punkt dla Grudnia.
    *
    */

   gdk_draw_rectangle(GTK_WIDGET(lookup_widget(frm_barline,
             "drawingarea1"))->window, get_gc("RoyalBlue"),
             TRUE, from_left - 2, from_top - 2, 3, 3);

}

gboolean load_data()
{
   conx = mysql_init(0L);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1,"problem w mysql_init ");
        return FALSE;
     }
   /* kontynuacja... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_init ok");

   conx = mysql_real_connect(conx, SERVER, LOGIN, PASS, DB, 0, 0L, 0);

   if (conx == 0L)
     {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1," problem mysql_real_connect ");
        return FALSE;
     }
   /* kontynuacja... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_real_connect ok");

   sql = g_strconcat("select * from tbl_obroty ",
                     "  order by miesiac", 0L);

   g_print("sql  %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
        gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1,"problem z mysql_query ");
        g_print("problem z mysql_query w load_data.\n");
        mysql_close(conx);
        return FALSE;
      }
   /* kontynuacja... */

   gtk_statusbar_push(GTK_STATUSBAR(lookup_widget(frm_barline,
                      "statusbar1")), 1, "mysql_query ok");

   result_set = mysql_store_result(conx);



   while (( row = mysql_fetch_row (result_set)) != 0)
     {
        static gint i = 0;
	/* Konwersja wartosci zwroconych w wynimu
	 * zapytania sql do liczb calkowitych, 
	 * wypelnienie tablic wartosciami
	 */

        month_bar_amounts[i] = atoi(row[2]);
        month_line_amounts[i] = atoi(row[3]);

        g_print("dane to  %d i %d\n", month_bar_amounts[i],
                                        month_line_amounts[i]);

        i++;
     }

   mysql_close(conx);

   return TRUE;

}
