/* listing 6.1

   
*/

use sesi;

drop table if exists tbl_towary;

create table tbl_towary
(
	towar 		varchar(8) 	NOT NULL      PRIMARY KEY,
	opis 	varchar(50) 	NOT NULL,
	cena		decimal(4,2)	NOT NULL
);

/* W ponizszej instrukcji LOAD TABLE (oraz kolejnych instrukcjach
    ladujacych dane w tym pkliku) nalezy podac nazwe pliku z danymiwraz
   z pelna sciezka dostepu.  Jesli nie podamy sciezki dostepu, to serwer MySQL
   domyslnie bedzie poszukiwal pliku w katalogu bazy danych, a nie w katalogu
   biezacym badz w katalogu projektu.  Nie zaleca sie umieszczania plikow tekstowych
   z danymi w katalogu bazy danych - tego katalogu nie nalezy
   zanieczyszczac zbednymi plikami.

  Zwrocmy rowniez uwage na fakt, ze pliki tekstowe posiadaja dane
   rozmieszczone w polach i kolumnach oddzielonych odstepami.
  W ponizszej instrukcji korzystamy z pliku new_master_item.txt,
  ktory zostal utworzony w innej aplikacji (najprawdopodobniej w
  arkuszu kalkulacyjnym) i przekopiowany w odpowiednie miejsce tak,
  aby instrukcja LOAD DATA INFILE zadzialala poprawnie. Plik zawiera
  liste towarow sprzedawanych przez firme, dla ktorej tworzymy aplikacje.

  Nalezy pamietac o wprowadzeniu poprawnej sciezki dostepu do pliku -
  w tym przypadku jest to , ale w innych komputerach najprawdopodobniej
  bedzie inna.
*/

LOAD DATA INFILE "/pliki/new_item_master.txt"
	INTO TABLE tbl_towary;

/* Sprawdzmy, czy import danych powiodl sie. W tabeli powinno znalezc
   sie okolo 600 rekordow. Dokladna liczba towarow jest podana w pliku
   tekstowym */

select count(*) from tbl_towary;

drop table if exists tbl_klienci;

create table tbl_klienci
(
	numer		smallint	NOT NULL	PRIMARY KEY,
	nazwa		varchar(100)	NOT NULL,
	dost_adr1	varchar(100),
	dost_adr2	varchar(100),
	dost_miasto	varchar(35),
	dost_stan	char(2),
	dost_kod	varchar(10),
	rach_adr1	varchar(100),
	rach_adr2	varchar(100),
	rach_miasto	varchar(35),
	rach_stan	char(2),
	rach_kod	varchar(10),
	kontakt_imie	varchar(50),
	kontakt_nazw	varchar(50),
	tel		varchar(12),
	stanowisko		varchar(50),
	uwagi	        varchar(255)
);
/*W ponizszej instrukcji LOAD DATA INFILE skorzystano z pliku tekstowego,
  wygenerowanego przez inny program i umieszczonego w odpowiednim
  katalogu. Plik zawiera dane klientow firmy.  */

LOAD DATA INFILE "/pliki/cust_mast.txt"
	INTO TABLE tbl_klienci;

/* Tym razem rowniez sprawdzimy poprawnosc zaladowania danych.
   W bazie powinno znalezc sie okolo 170 rekordow. Najlepiej
   sprawdzic dokladna liczbe klientow w pliku zrodlowym
*/

select count(*) from tbl_klienci;

/* Teraz zmodyfikujemy strukture tabeli tbl_klienci tak, aby  w pierwsza kolumna
   miala atrybut AUTO_INCREMENT. W pliku z danymi cust_mast.txt
   pierwszym polem jest numer klienta, ktory pelni role klucza glownego
   tabeli. W bazie danych MySQL takie pole powinno miec atrybut auto_increment.
   Jesli tworzymy tabele, ktorej pierwsze pole ma atrybut auto_increment,
   to w trakcie ladowania danych moze wystapic problem, jezeli
   pierwszy klient w pliku nie ma identyfikatora 1. Dlatego tez, aby uniknac
   tego typu problemu podczas wykonywania instrukcji LOAD DATA INFILE, lepiej
   odpowiednio zmodyfikowac pole numer po zaladowaniu danych poczatkowych, .
   a nie w trakcie tworzenia tabeli.

   Zwrocmy uwage na to, ze nie musimy jeszcze raz okreslac atrybutu PRIMARY KEY
   dla pola numer. Jesli w instrukcji ALTER TABLE ponownie zadeklarujemy ten atrybut,
   MySQL wygeneruje komunikat o wystapieniu bledu zwiazanego z duplikowaniem sie
   klucza glownego w tabeli.

   Jeszcze jeden, ostatni komentarz.  Jesli dowolny plik, ktory ma byc wykorzystany
   przez MySQL, konczy sie komentarzem, to proba zaladowania danych do bazy
   danymi z takiego pliku zakonczy sie bledem, bo MySQL wymaga, aby ostatnim znakiem
   pliku byl srednik.
*/

ALTER TABLE tbl_klienci
MODIFY numer	SMALLINT
		NOT NULL
		AUTO_INCREMENT;


