#include <mysql.h>
#include <gtk/gtk.h>

/* Biblioteka stdlib.h zawiera funkcje  atof.
 * Biblioteka string.h jest potrzebna dla funkcji przeszukujacych i jest
 *          wykorzystywana w trakcie formatowania tekstu.
 * Biblioteka time.h   jest potrzebna do operacji zapisu do pliku.
 * Biblioteka stdio.h  jest rowniez potrzebna przy zapisie do pliku.
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdio.h>

#include "support.h"

/********** Zmienne globalne *****************/

GtkWidget *frm_main;
GtkWidget *frm_items_ordered;
GtkWidget *frm_find_item;
GtkWidget *frm_find_customer;

/* conx reprezentuje polaczenie z baza danych;  ma zasieg
 * globalny, dzieki czemu wszystkie funkcje maja dostep
 * do bazy danych "sesi".
 */

MYSQL      *conx;

/* dbl_order_total jest zmienna globalna.
 * Mozna z niej korzystac bez koniecznosci
 * jawnej konwersji danych pomiedzy gchar/gfloat/gdouble
 *
 */

gdouble    dbl_order_total = 0;

/* Zmienna need_to_save_edits okresla, czy
 * zmieny dokonano w dowolnym obiekcie tekstowym
 * wyswietlajacym zawartosc tabeli
 * tbl_klienci.  Jej wartosc bedzie sprawdzana czesto
 * w wielu miejscach programu, dlatego tez najrozsadniej
 * bedzie utworzenie jej jako zmiennej globalnej.
 */

gboolean   need_to_save_edits;

/********** Prototypy funkcji **************/

void    connect_to_db();
void    fatal_msgbox(gchar *msg);
void    get_conx();
GList   *get_customer_numbers();
void    fill_customer_info();
void    clear_frm_main();
void    fill_frm_main(MYSQL_ROW in_row);
void    speed_add();
void    slow_add();
void    remove_ordered_item();
void    select_item(gchar *target_item_num);
void    enter_found_items(gchar *str);
void    select_customer(gchar *target_customer_num);
void    enter_found_customers(gchar *str);
void    write_order();
gchar   *right_pad(gchar *in_str, gchar *pad_char, gint final_length);
gchar   *left_pad(gchar *in_str, gchar *pad_char, gint final_length);
void    create_new_customer();

/********** Funkcje****************/
void connect_to_db()
{
   /* Listing 7.1
    *
    * Ta funkcja sluzy do ustanowienia polaczenia z baza danych
    * "sesi".  Okresla wartosc zmiennej
    * conx, ktora reprezentuje (globalnie) polaczenie z
    * baza danych.
    *
    * 0L jest rownowazna wartosci NULL i jest bardziej "przenosna".
    * W programie przeznaczonym tylko dla systemu Linux
    * mozna skorzystac z wartosci NULL.
    */

   g_print("Ustanawianie polaczenia z baza danych...\n");

   conx = mysql_init ((MYSQL *)0L);
   if (conx == 0L)
      {
        /* Co program ma zrobic, gdy polaczenie nie zostanie nawiazane?
         * Kontynuacja programu bez ustanowionego
         * po??czenia z baza MySQL nie ma sensu
         * W wiekszej aplikacji mozna by umiescic okno
         * dialogowe z informacja o blednym logowaniu
         * i pomoca w rozwiazaniu tego problemu.
         * W naszej aplikacji nie beedziemy umieszczac systemu
         * rozwiazywania problemow z polaczeniem
         *
         * Gdy wystapi blad krytyczny, program wyswietli
         * odpowiedni komunikat uzytkownikowi i zostanie zamkniety.
         */

        g_print("Blad w mysql_init...\n");

        /* Funkcje fatal_msgbox() zdefiniowano w kolejnym listingu.
         * Z tej funkcji bedziemy korzystac zawsze wtedy, gdy
         * wystapi blad krytyczny, ktory nie pozwoli na kontynuacje programu.
         */

        fatal_msgbox("Inicjacja bazy danych nie powiodla sie,\n"
                     " skontaktuj sie z administratorem...");
      }

   /* Jesli ponizsza funkcje wywolamy bez parametru oznaczajacego
    * nazwe bazy danych (w tym przypadku "sesi"), to funkcja
    * zwroci wartosc OK, a nie blad.  Powodem takiego zachowania sie
    * funkcji jest fakt, ze jej glownym zadaniem jest nawiazanie polaczenia
    * z serwerem, a nie z okreslona baza. Jesli z jakiegos powodu nie polaczymy sie
    * z baza, mozemy nawet o tym nie wiedziec az do chwili wykonania
    * zapytania, ktore powinno zwrocic jakas informacje.
    */

   conx = mysql_real_connect (conx, "localhost", "sesi_user", "aaa", "sesi", 0, 0L, 0);
   if (conx == 0L)
      {
	g_print("Blad w mysql_real_connect...\n");

        /* Kolejne funkcje sa przykladem obslugi bledow,
         * ktore moga pojawic sie w trakcie dzialania programu.
         * Skorzystamy tu z funkcji obslugi bledow MySQL.  Informacje
         * pochodzace z tych funkcji moga byc przydatne w sledzeniu bledow,
         * i ich usuwaniu, wiec powinnismy z tych funkcji skorzystac w programie.
         */

        g_print("Numer bledu %i...\n", mysql_errno(conx));
        g_print("Opis bledu %s...\n", mysql_error(conx));

        /* Jesli funkcja mysql_real_connect zwroci blad, to
         * kontynuacja dzialania programu nie ma sensu.
         */

        fatal_msgbox(g_strconcat("Nie mozna nawiazac polaczenia",
                                 " z baza danych.\n\n",
                                 "Numer bledu: ",
                                 g_strdup_printf("%d", mysql_errno(conx)),
                                 "\nOpis bledu: \n",
                                 g_strdup_printf("%s", mysql_error(conx)),
                                 0L)
                    );
      }

   g_print("Polaczenie z baza danych nawiazane...\n");

   /* Ostatnim etapem nawiazywania polaczenia jest wykonanie zapytania
    * ktore potwierdzi, ze jestesmy polaczeni z wlasciwa baza.
    */

   if (mysql_query (conx, "select count(*) from tbl_klienci") != 0)
      {
                fatal_msgbox(g_strconcat("Nie mozna polaczyc sie z ",
                                         "baza danych SESI.\n\n",
                                         "Numer bledu:  ",
                                         g_strdup_printf("%d",
                                                  mysql_errno(conx)),
                                         "\nOpis bledu:  ",
                                         g_strdup_printf("%s",
                                                  mysql_error(conx)),
                                         0L)
                );

	        g_print("Nie mozna polaczyc sie z wlasciwa baza danych...\n");
      }
   else g_print("Polaczono z baza \"sesi\"...\n");

   /* Pozostalo poinformowanie uzytkownika o prawidlowym wykonaniu polaczenia.  */

   }
void fatal_msgbox(gchar *msg)
{
   /* Listing 7.2
    *
    * Ta funkcja jest wywolywana zawsze wtedy, gdy wystapi blad krytyczny
    * programu. Okna dialogowe sa kiepskim sposobem komunikacji z
    * uzytkownikiem, ale czasem przydatnym. Gdy wystapi blad krytyczny
    * programu, to mozna skorzyatc z okna dialogowego, aby zamiescic
    * komunikat i wyjasnic, co sie stalo i dlaczego program zostanie zamkniety,
    * a takze informacje jaka nalezy przekazac administratorowi
    * Czasem wykorzystuje sie okna dialogowe po to, aby uzytkownik mogl podjac
    * decyzje o dalszym przebiegu dzialania programu
    *
    * W funkcji wywolano gtk_dialog_new(), aby utworzyc pojemnik okna
    * dialogowego, zawierajacego etykiete i przycisk OK.
    */

   GtkWidget *msgbox;
   GtkWidget *lbl;
   GtkWidget *cmd_ok;

   msgbox = gtk_dialog_new();
   lbl = gtk_label_new(g_strconcat("Wystapil blad krytyczny.\n\n",
                                   msg,"\n\n",
                                   "Prosze zanotowac",
                                   "ponizsze informacje.\n\n",
                                   "Klikniecie na OK zakonczy  ",
                                   "dzialanie programu",
                                   0L) );

   cmd_ok = gtk_button_new_with_label("Ok");

   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(msgbox)->vbox),
                              lbl, FALSE, FALSE, FALSE);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(msgbox)->action_area),
                              cmd_ok, FALSE, FALSE, FALSE);

   /* Zwrocmy uwage na wywolania ponizszych funkcji.  Druga z nich
    * jest juz nam znana, ale zauwazmy, ze zamiast tworzyc osobna funkcje zwrotna
    * i w niej wywolac gtk_main_quit(), funkcja zamykajaca aplikacje
    * jest wywolywana bezposrednio.
    *
    * Pierwsza z nich, gtk_signal_connect_object(), jest wykorzystywana wtedy,
    * gdy sygnal pochodzi z obiektu GtkObject.  Zwrocmy uwage, ze obie
    * funkcje daja taki sam efekt.  Pierwsza z nich umiescilismy na wszelki
    * wypadek, bonie mamy pewnosci, ze uzytkownik kliknie na przycisk "Ok".
    * Uzytkownik mo?e zamknac nasze okno za pomoca przycisku "X" umieszczonego
    * w lewym, gornym rogu okna.  Bez wywolania zdarzenia delete_event
    * uzytkownik powrocilby do glownego okna aplikacji, ale nie moglby juz
    * nic zrobic.
    */
   gtk_signal_connect_object(GTK_OBJECT(msgbox),
                             "delete_event",
                             GTK_SIGNAL_FUNC(gtk_main_quit),
                             GTK_OBJECT(msgbox) );

   gtk_signal_connect(GTK_OBJECT(cmd_ok),
                      "clicked",
                      GTK_SIGNAL_FUNC(gtk_main_quit),
                      0L);

   gtk_widget_show_all(msgbox);
   getchar();

}
void get_conx()
{
   /* Ta funkcja odswierza polaczenie z baza danych w razie
    * potrzeby.  Nie zawiera obslugi bledow; poprawnosc
    * nawiazywania polaczenia zostala ju? raz sprawdzona na poczatku,
    * wiec mo?na zalozyc, ze tym razem nawiazanie polaczenia jest
    * prawie pewne.
    */

   conx = mysql_init (0L);
   /* W ponizszej funkcji root oznacza uzytkonika systemu, ktry
    * zainstalowal pakiet MySQL za pomoca plikow RPM. Je?li instalacje
    * przeprowadzil inny uzytkownik, to nale?y wpisac jego nazwe (zamiast root).*/
   mysql_real_connect (conx, "localhost", "sesi_user", "aaa", "sesi", 0, 0L, 0);
}

GList *get_customer_numbers()
{
   /* Ta funkcja sluzy do pobrania listy z numerami klientow
    * z bazy danych i umieszczenia jej w obiekcie cbo_customer_numbers
    * znajdujacym sie w oknie frm_main.*/

   GList       *list_of_customers = 0L;
   MYSQL_RES   *result_set;
   MYSQL_ROW   row;

   get_conx();

   /* Gdy okno frm_main zostanie otwarte, w polu listy rozwijanej znajdzie
    * si? domyslnie pierwszy element.
    */

   if (mysql_query (conx, "select distinct(numer) from tbl_klienci") != 0)
      {
                fatal_msgbox(g_strconcat("Nie mozna pobrac listy",
                                         " z numerami klientow.\n\n",
                                         "Numer bledu:  ",
                                         g_strdup_printf("%d",
                                                 mysql_errno(conx)),
                                         "\nOpis bledu:  ",
                                         g_strdup_printf("%s",
                                                 mysql_error(conx)),
                                         0L)
                );

         g_print("Nie mozna pobrac listy klientow..\n");
      }
   else g_print("Pobrano identyfikatory klientow z bazy danych...\n");

   /* Teraz b?dziemy przemieszczac si? wiersz po wierszu i odczytywac numery klientow
    * a nastepnie dodawac je do listy cbo_customer_number.
    */

   result_set = mysql_store_result (conx);
   while ((row = mysql_fetch_row (result_set)) != 0L)
      { list_of_customers = g_list_append(list_of_customers, row[0] );
      }

   /* Je?li umiescimy wartosc Nowy na koncu listy, to uzytkownik
    * b?dzie mogl uzywac strzalek gora-dol i przeszukiwac rekordy
    * bez wybierania wartosci "Nowy".  Dodawanie nowego klienta bedzie
    * wykonywane sporadycznie, a uzytkownik bedzie mogl od razu przejsc
    * na koniec listy do elementu "Nowy" albo wpisac jego nazwe.
    */

   list_of_customers = g_list_append(list_of_customers, "New");

   return list_of_customers;
}
void fill_customer_info()
{

   /* Ta funkcja sluzy do odczytu rekordu klienta z bazy danych
    * i wprowadzenia ich w odpowiednich miejscach okna frm_main
    * za pomoca funkcji fill_frm_main */
   GtkCombo    *cbo;
   MYSQL_RES   *result_set;
   MYSQL_ROW   row;
   /* Najpierw nale?y nawiazac polaczenie z baza danych, a nastepnie pobrac
    * numer klienta z potomnego widgeta pola cbo_customer_number */
   get_conx();

   cbo = GTK_COMBO(lookup_widget(frm_main, "cbo_customer_number"));
   g_print(gtk_entry_get_text(GTK_ENTRY(cbo->entry)));
   g_print("\n");
    /* Sprawdzmy najpierw, czy uzytkownik nie dodaje nowego klienta
    * a dopiero potem wykonajmy odpowiednie zapytanie. Je?li uzytkownik
    * tworzy nowy rekord, to wyczyscimy pola okna frm_main i wywolamy
    * funkcje create_new_customer()*/
   if (g_strcasecmp(gtk_entry_get_text (GTK_ENTRY(cbo->entry)), "New") == 0)
       {
            g_print("Tworzenie nowego rekordu uzytkownika...\n");
            clear_frm_main();

            create_new_customer();

            /* Po ustanowieniu nowego rekordu - wychodzimy z funkcji. */
            return;
       }
   /* Jesli w polu cbo_customer_number nie wpisano zadnej wartosci,
    * to nalezy wyczyscic wszystkie pola okna.
    */
   if (g_strcasecmp(gtk_entry_get_text (GTK_ENTRY(cbo->entry)), "") == 0)
       {
            g_print("Brak danych:  Rekord klienta pusty...\n");
            clear_frm_main();

            return;
       }

   /* mysql_query() zwraca wynik zawsze; je?li przeslane zapytanie by?o
    * prawidlowe, to otrzymamy pozadane informacje. Je?li zapytanie
    * zawiera numer nieistniejacego klienta, to funkcja nie zwroci
    * zadnego rekordu.
    */

   if (mysql_query (conx,
                    g_strconcat("select * ",
                                "from tbl_klienci where numer = ",
                                gtk_entry_get_text (GTK_ENTRY(cbo->entry)),
                                0L
                                )
                   ) != 0
      )
     {
        g_print("blad mysql_query w fill_customer_info()...\n");
         }
   else
     {
        g_print("Pobieranie danych ...\n");

        result_set = mysql_store_result (conx);

        if (mysql_num_rows (result_set) == 0)
        {
           g_print("Bledny numer klienta...\n");

           /* Wyczyszczenie frm_main. */
           clear_frm_main();

        }
        else
        {
          /* Moznaby sprawdzic dodatkowo, czy baza nie zwrocila wiecej
           * ni? jeden rekord, co oznaczaloby, ze pole tbl_klienci.numer
           * jest nieoprawidlowe (powinno zawierac unikatowe dane).
           *
           * Klucz glowny okreslilismy ju? podczas tworzenia bazy danych
           * wiec - dla uproszczenia tej procedury - nie b?dziemy
           * sprawdzac unikatowosci klucza.
           */

          row = mysql_fetch_row (result_set);
          g_print("Przygotowanie do wypelnienia widgetow wartosciami...\n");

          /* Wypelnimy frm_main danymi o kliencie.  Przesylanym parametrem
           * jest rekord danych z bazy sesi.
           */

          fill_frm_main(row);

        }

     }
   return;

}

void clear_frm_main()
{
    g_print("Czyszczenie informacji o kliencie...\n");

    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                            "entry_customer_name")), "");

    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_ship_to_addr1")), "" );
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_ship_to_addr2")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_ship_to_city")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_ship_to_st")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_ship_to_zip")), "");

    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_bill_to_addr1")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                                             "entry_bill_to_addr2")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                                             "entry_bill_to_city")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                                             "entry_bill_to_st")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                                             "entry_bill_to_zip")), "");

    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, "entry_first")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, "entry_last")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, "entry_title")), "");
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, "entry_phone")), "");

    /* Usuniemy tekst z pola tekstowego, przeznaczonego na uwagi klienta.
     * Uwagi dotyczace zamowienia powinny pozostac bez wzgl?du na zmiane 
     * klienta, bo dotycza egzemplarza aplikacji, a nie okreslonego.
     * klienta*/

    gtk_text_set_point(GTK_TEXT(lookup_widget(frm_main,
                                      "txt_customer_comments")), 0);
    g_print("current insertion point is %i\n...", 
            gtk_text_get_point(GTK_TEXT(lookup_widget(frm_main,
                                       "txt_customer_comments"))));

    g_print("delete returned %i\n",
              gtk_text_forward_delete (GTK_TEXT(lookup_widget(frm_main, 
                                                   "txt_customer_comments")), 
                          gtk_text_get_length(GTK_TEXT(lookup_widget(frm_main, 
                                                  "txt_customer_comments")))  
             ));
}

void fill_frm_main(MYSQL_ROW in_row)
{   /* Ta funkcja sluzy do wypelnienia pol tekstowych frm_main przeznaczonych
     * na dane o kliencie 
     * Wyczyscimy okno, aby operacja wypelniania pol rozpoczynala si? od okreslonego
     * stanu.
     */

    clear_frm_main();
   /* in_row jest parametrem funkcji. Zawiera jeden rekord pochodzacy z tabeli
    * tbl_klienci, ktry stanowi pozadana informacje. */
     
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                        "entry_customer_name")), in_row[1]);

    if (in_row[2] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_ship_to_addr1")), in_row[2]);
    }
    if (in_row[3] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_ship_to_addr2")), in_row[3]);
    }
    if (in_row[4] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_ship_to_city")), in_row[4]);
    }
    if (in_row[5] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_ship_to_st")), in_row[5]);
    }
    if (in_row[6] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_ship_to_zip")), in_row[6]);
    }

    if (in_row[7] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_bill_to_addr1")), in_row[7]);
    }
    if (in_row[8] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_bill_to_addr2")), in_row[8]);
    }
    if (in_row[9] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main,
                          "entry_bill_to_city")), in_row[9]);
    }
    if (in_row[10] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_bill_to_st")), in_row[10]);
    }
    if (in_row[11] != 0L ) {
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                          "entry_bill_to_zip")), in_row[11]);
    }

    g_print("Wypelnianie informacje o kontakcie z klientem...\n");
    if (in_row[12] != 0L ) {
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                           "entry_first")), in_row[12]);
    }

    if (in_row[13] != 0L ) {
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                           "entry_last")), in_row[13]);
    }

    if (in_row[14] != 0L ) {
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                           "entry_phone")), in_row[14]);
    }

    if (in_row[15] != 0L ) {
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frm_main, 
                           "entry_title")), in_row[15]);
    }
 
    if (in_row[16] != 0L ) {
        gtk_text_insert(GTK_TEXT(lookup_widget(frm_main,
                                  "txt_customer_comments")),
                        0L, 0L, 0L,
                        in_row[16], -1
                       );
    }
   /* Poniewaz otrzymane dane z bazy nie podlegly zadnej edycji,
    * przycisk cmd_save_edits nie jest nam potrzebny, wiec
    * go zdezaktywujemy. Aby to zrobic, wpiszemy wartosc false do
    * znacznika need_to_save_edits.*/
    need_to_save_edits = FALSE;
    gtk_widget_set_sensitive(lookup_widget(frm_main,
                                          "cmd_save_edits"), FALSE);
}
void fill_items_ordered()
{
   MYSQL_RES   *result_set;
   MYSQL_ROW   db_row;
   gchar       *clist_row[3] = {"", "", ""};

   /* Funkcja wprowadza poczatkowe wartosci do obiektow okna frm_items_ordered,
    * Zakladamy, ze funkcja bedzie wywolywana w wyniku wystapienia okreslonych
    * zdarzen pochodzacych z frm_items_ordered, bo lista dostepnych
    * towarow w magazynie (umieszczona w bazie danych) nie bedzie sie
    * zbyt czesto zmieniac.
    *
    * Kod zapewniajacy dostep do bazy danych bedzie dzialal podobnie, jak
    * w poprzednio omowionych funkcjach.
    */

   get_conx();
   if (mysql_query (conx, "select * from tbl_towary") != 0)
      {
          /* Co zrobic, jesli zapytanie nie zwroci odpowiedniej
           * listy towarow?  Teoretycznie, jesli uzytkownik
           * zna numer kazdego towaru, aplikacja powinna sprawnie
           * utworzyc odpowiednie zapytanie o poszukiwana informacje
           * (cene towaru).  Czasem moze cos sie jednak zdarzyc
           * w trakcie nawiazywania polaczenia z baza danych
           * i kazde zapytanie zakonczy sie niepowodzeniem.  Powinnismy,
           * "ubezpieczyc sie" na taka ewentualnosc.i zrobic cos innego
           */

    g_print("Blad odczytu listy towarow...\n");

          /* ...zamiast skorzystac tylko z fatal_msgbox().  Jesli faktycznie
           * nic nie da sie zrobic, polaczenie zostanie permanentnie zerwane
           * i nie bedzie mozna odczytac ceny towaru, to dopiero wtedy wywolamy 
           * fatal_msgbox().
           */
      }
   else g_print("Pobrano numery klientow z bazy danych...\n");

   /* Teraz bedziemy przechodzic rekord po rekordzie , odczytywac numery klientow
    * i dodawac je do obiektu cbo_customer_number.
    */

   result_set = mysql_store_result (conx);
   while ((db_row = mysql_fetch_row (result_set)) != 0L)
      {  clist_row[0] = db_row[0]; 
         clist_row[1] = db_row[1];
         clist_row[2] = db_row[2]; 
         gtk_clist_append(GTK_CLIST(lookup_widget(frm_items_ordered, 
                                                  "clist_items")),
                          clist_row);
      }

}

void speed_add()
{
   MYSQL_RES   *result_set;
   MYSQL_ROW   db_row;
   gchar       *sql;
   gchar       *clist_row[4] = {"", "", "", ""};
   gint        int_quantity;
   gchar       *str_quantity;
   gdouble     dbl_total_price;
   gchar       *str_total_price;
   gchar       *str_total_price_formatted;
   gchar       *str_order_total;
   gchar       *str_order_total_formatted;

   /* Funkcja bedzie wywolywana zawsze wtedy, gdy uzytkownik kliknie na cmd_add,
    * znajdujacy sie w lewym gornym rogu okna frm_items_ordered.  Ta funkcja zapewni "szybka"
    * metode wprowadzania towarow. Utworzymy jeszcze jedna funkcje, zapewniajaca "wolniejszy" 
    * sposob dodawania towarow - slow_add (ktora omowiono jako nastepna funkcje).

    * Najpierw nalezy pobrac numer towaru i jego cene z bazy danych
    */

   get_conx();
   sql = g_strconcat("select * from tbl_towary where towar = '",
                      gtk_editable_get_chars(
                            GTK_EDITABLE(lookup_widget(frm_items_ordered,
                                                       "entry_item_number"
                                )) , 0, -1), "'", 0L);

   g_print("sql %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Blad odczytu informacji przez mysql_query...\n");
      }
   else
      {
          result_set = mysql_store_result (conx);

          /* W tym miejscu jestesmy pewni, ze zapytanie sotalo wykonane poprawnie
           * ale nie mamy jeszcze pewnosci, czy towar o wprowadzonym numerze
           * istnieje.  Podobnie, jak w fill_customer_info(), sprawdzimy
           * czy liczba rekordow jest wieksza od zera 0.
           */

           if (mysql_num_rows (result_set) == 0)
           {
               g_print("Nieprawidlowy kod towaru...\n");

               /* W tym miejscu uzytkownik powinien dostac informacje, ze
                * nic nie dodano do obiektu CList.  Zakladamy,
                * ze w projekcie wiekszym od tworzonego powinnismy
                * utworzyc jakies dodatkowe okno informacyjne,
                * ale tutaj wystarczy informacja na pasku stanu.
                */

            }
            else
            {
                g_print("Odczytano informacje o towarze...\n");

                /* Po weryfikacji numeru towaru, pobierzemy liczbe
                 * zamawianych sztuk, wykonamy odpowiednie obliczenia,
                 * i dodamy dane do obiektu clist_items_ordered.
                 */

                db_row = mysql_fetch_row (result_set);

                /* Ponizej pokazano, jak z przewijanego pola odczytac te
                 * sama informacje jako dane roznego typu.
                 */

                int_quantity = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
                                     lookup_widget(frm_items_ordered,
                                     "spinbutton_quantity")));
                str_quantity = gtk_editable_get_chars(
                                   GTK_EDITABLE(
                                     lookup_widget(frm_items_ordered,
                                     "spinbutton_quantity")
                                   ), 0, -1);

                dbl_total_price =  int_quantity * atof(db_row[2]);
                g_print("dbl_total_price is %f\n", dbl_total_price);

                str_total_price = g_strdup_printf("%f", dbl_total_price);

                clist_row[0] = db_row[0];
                clist_row[1] = db_row[1];
                clist_row[2] = str_quantity;

                /* Nastepnie, odpowiednio sformatujemy wyswietlane dane,
                 * odnajdziemy wartosc po przecinku ",xx".
                 */

                str_total_price_formatted = g_strndup(str_total_price,
                                            strcspn(str_total_price, ",") + 3);

                clist_row[3] = str_total_price_formatted;

                gtk_clist_append(GTK_CLIST
                              (lookup_widget(frm_items_ordered,
                                             "clist_items_ordered")),
                                 clist_row);

                /* Na koniec jeszcze raz podliczymy sume zamowienia i umiescimy
                 * w etykiecie. Latwiej caly czas dokonywac obliczen, niz
                 * odnosic sie do wartosci obiektu clist.
                 */

                dbl_order_total = dbl_order_total + dbl_total_price;

                str_order_total = g_strdup_printf("%f", dbl_order_total);
                str_order_total_formatted = g_strndup(str_order_total,
                                            strcspn(str_order_total, ".") +3);

                gtk_label_set_text(GTK_LABEL(lookup_widget
                                               (frm_items_ordered,
                                                "lbl_order_total_numeric")),
                                   str_order_total_formatted
                                  );

            }
      }
}
void slow_add()
{
   /* Ta funkcja bedzie wykorzystywana do dodawania towarow za
    * pomoca elementow prawej czesci okna zamowienia. Ta metoda bedzie wolniejsza,
    * bo uzytkownik musi czesciej korzystac z myszy. Funkcja zostanie wywolana,
    * gdy uzytkownik kliknie na przycisk Dodaj, znajdujacy sie pomiedzy
    * dwiema listami w oknie frm_items_ordered.*/
   GtkCList    *clist_target;
   gint        row_target = -1;

   gchar       *cell_item_number;
   gchar       *cell_item_description;
   gchar       *cell_item_price;

   gchar       *clist_row[4] = {"", "", "", ""};
   gint        int_quantity;
   gchar       *str_quantity;
   gdouble     dbl_total_price;
   gchar       *str_total_price;
   gchar       *str_total_price_formatted;
   gchar       *str_order_total;
   gchar       *str_order_total_formatted;

   clist_target = GTK_CLIST(lookup_widget(frm_items_ordered, "clist_items"));

   /* Tu pobieramy wartosc z zanzaczonego wiersza.
    * Wartosc 0 odpowiada zerowemu elementowi listy.
    */

   row_target = (gint)g_list_nth_data( (clist_target)->selection, 0 );

   g_print("Wiersz do przeniesienia to%i...\n", row_target);

   if (row_target == -1)
      {


         g_print("Nie wybrano towaru...\n");

         return;
      }
   /* Kolejne trzy funkcje sa wywolane w celu odnalezienia
    * informacji dotyczacych wybranych towarow w CList_items.
    * Pobierane sa: numer towaru,opis i cena, ktora bedzie wykorzystana w dalszym
    * fragmencie programu.
   */
   gtk_clist_get_text(GTK_CLIST(lookup_widget(frm_items_ordered,
                      "clist_items")),
                      row_target, 0, &cell_item_number);
   gtk_clist_get_text(GTK_CLIST(lookup_widget(frm_items_ordered,
                      "clist_items")),
                      row_target, 1, &cell_item_description);
   gtk_clist_get_text(GTK_CLIST(lookup_widget(frm_items_ordered,
                      "clist_items")),
                      row_target, 2, &cell_item_price);


   /* Spinbutton1 jest polem przewijanym, umieszczonym obok cmd_add_down, miedzy
    * dwoma obiektami CList.  Zapomnielismy mu zmienic nazwe... :-|
    */

   int_quantity = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(
                                     lookup_widget(frm_items_ordered,
                                     "spinbutton_quantity_down")));
   str_quantity = gtk_editable_get_chars(
                                   GTK_EDITABLE(
                                     lookup_widget(frm_items_ordered,
                                     "spinbutton_quantity_down")
                                   ), 0, -1);
   /* Wyznaczymy wrtosc zamowienia przez pomnozenie liczby zamawianych
    * sztuk przez cene jednostkowa. Nastepnie przygotujemy tablice CList_row[]
    * ustalajac dane, ktore maja byc wprowadzonedo obiektu CList okna zamowienia.
   */
    dbl_total_price =  int_quantity * atof(cell_item_price);
    g_print("dbl_total_price is %f\n", dbl_total_price);

    str_total_price = g_strdup_printf("%f", dbl_total_price);

    clist_row[0] = cell_item_number;
    clist_row[1] = cell_item_description;
    clist_row[2] = str_quantity;
   /* Sformatowalismy odpowiednio dane (pierwsza funckja), a nastepnie wprowadzimy
    * sformatowana wartosc do ostatniego pola tablicy. po czym od razu
    * zawartosc tablicy clist_row[] wprowadzimy do clist_items_ordered.
   */
    str_total_price_formatted = g_strndup(str_total_price,
                                          strcspn(str_total_price, ".") + 3);

    clist_row[3] = str_total_price_formatted;

    gtk_clist_append(GTK_CLIST
                       (lookup_widget(frm_items_ordered,
                                      "clist_items_ordered")),
                     clist_row);

    /* Ponowne obliczenie wartosci zamowienia. */

    dbl_order_total = dbl_order_total + dbl_total_price;
    str_order_total = g_strdup_printf("%f", dbl_order_total);
    str_order_total_formatted = g_strndup(str_order_total,
                                          strcspn(str_order_total, ".") +3);

    gtk_label_set_text(GTK_LABEL(lookup_widget
                                      (frm_items_ordered,
                                       "lbl_order_total_numeric")),
                       str_order_total_formatted
                       );

}
void remove_ordered_item()
{
   /* Funkcja sluzy do usuniecia towaru z obiektu clist_items_ordered,
    * ktory zostal omylkowo dodany przez pracownika przyjmujacego
    * zamowienia. Po usunieciu towaru z listy
    * trzeba odpowiednio zmniejszyc wartosc zamowienia.
   */
   GtkCList  *clist_target;
   gint      row_target = -1;
   gchar     *cell_line_item_price;

   clist_target = GTK_CLIST(lookup_widget(frm_items_ordered,
                                          "clist_items_ordered"));

   /* Ponizsza funkcja pobiera dane wiersza zaznaczonego.
    * 0 odpowiada zerowemu elementowi listy.
    */

   row_target = (gint)g_list_nth_data( (clist_target)->selection, 0 );

   g_print("Usuwany rekord %i...\n", row_target);

   if (row_target == -1)
      {

         g_print("Nie wybrano zadnego elementu z clist_items_ordered...\n");

         return;
      }

   /* ... kontynuacja usuwania elementu... */

   /* Najpierw nalezy pobrac liczbe sztuk danego towaru, aby
    * odpowiednio odliczyc koszt towaru od wartosci zamowienia.
    */

   gtk_clist_get_text(GTK_CLIST(lookup_widget(frm_items_ordered,
                      "clist_items_ordered")),
                      row_target, 3, &cell_line_item_price);

   dbl_order_total = dbl_order_total - atof(cell_line_item_price);

   gtk_label_set_text(GTK_LABEL(lookup_widget
                                      (frm_items_ordered,
                                       "lbl_order_total_numeric")),
                        g_strdup_printf("%f", dbl_order_total)
                       );

   /* Na koniec trzeba usunac niepozadany towar. */

   gtk_clist_remove(clist_target, row_target);
}
void select_item(gchar *target_item_num)
{
   /* Ta funkcja jest przeznaczona dla okna frm_items_ordered
    * - po odnalezieniu towaru w bazie danych (przez odpowiedni widget),
    * a wlasciwie jego numeru, funkcja ma za zadanie jego zaznaczenie
    * w liscie clist_items.  Gdy jako parametr podamy numer towaru (
    * jedyny parametr tej funkcji), funkcja przeszuka po kolei
    * wszystkie elementy listy i znajdzie odpowiadajacy podanemu parametrowi
    * matches the input parameter.
    */

   GtkCList    *target_clist;
   gint        number_of_rows;
   gint        counter;
   gchar       *clist_item_number;

   g_print("Numer elementu poszukiwanego:  %s...\n", target_item_num);

   target_clist = GTK_CLIST(lookup_widget(frm_items_ordered, "clist_items"));

   /* Najpierw trzeba dowiedziec sie, ile jest rekordow w liscie.
    */

   number_of_rows = ((target_clist)->rows);

   g_print("liczba rekordow:  %i", number_of_rows);

   /* Przeszukiwanie listy w celu odnalezenia wlasciwego rekordu. */

   for(counter = 0; counter < number_of_rows; counter++)
    {
         gtk_clist_get_text(GTK_CLIST(lookup_widget(
                                         frm_items_ordered,
                                         "clist_items")),
                      counter, 0, &clist_item_number);

         if (g_strcasecmp(clist_item_number, target_item_num) == 0)
            {
               g_print("Znaleziono poszukiwany element %i w clist_items..\n", counter);
               break;
            }
         else
            {
               /* kontynuacja przeszukiwania... */
            }

    }

   /* Po odnalezieniu poszukiwanego elementu trzeba go zaznaczyc
    * i pokazac w oknie.
    */

   gtk_clist_select_row(target_clist, counter, 0);
   gtk_clist_moveto(target_clist, counter, 0, 0, 0);

}
void enter_found_items(gchar *str)
{
   /* Ta funkcja sluzy do przeszukiwania tabeli tbl_towary w celu
    * odnalezienia pasujacych elementow do tekstu podanego przez
    * uzytkownika (str jets parametrem przekazywanym tej
    * funkcji). Odnalezione rekordy zostana wyswietlone w
    * clist_found_items
    *
    * Najpierw trzeba polaczyc sie z baza danych i odszukac
    * pasujace elementy.
    *
    * Pamietajmy, ze funkcja poszukuje lancucha znakow (
    * parametru wejsciowego) w kolumnach nie numerycznych.
    * Bedziemy przeszukiwac wszystkie pola tekstowe (tabeli).
    *
    */

   MYSQL_RES   *result_set;
   MYSQL_ROW   db_row;
   gchar       *sql;
   gchar       *clist_row[3] = {"", "", ""};
   gint        counter;
   gint        number_of_rows;

   g_print("rozpoczecie create_find_item_clist...\n");

   get_conx();

   /* W MySQL, znak procentu jest znakiem zastepczym dla
    * wszytskich pasujacych elementow.
    */

   sql = g_strconcat("select * from tbl_towary where towar like '%",
                     str,
                     "%' or opis like '%",
                     str,
                     "%'",
                     0L);

   g_print("sql is :  %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Blad odczytu danych z mysql_query...\n");
      }
   else
      {
          result_set = mysql_store_result (conx);
          db_row = mysql_fetch_row (result_set);

          /* Wyczyszczenie zawartosci obiektu clist. */
          gtk_clist_clear(GTK_CLIST(lookup_widget(frm_find_item, 
                         "clist_found_items")));

          number_of_rows = mysql_num_rows(result_set);
          g_print("liczba rekordow:  %i", number_of_rows);
 
          for (counter = 0; counter < number_of_rows; counter++)
            { 
                  clist_row[0] = db_row[0];
                  clist_row[1] = db_row[1];
                  clist_row[2] = db_row[2];

                  gtk_clist_append(GTK_CLIST(lookup_widget(frm_find_item, 
                                   "clist_found_items")),
                                   clist_row);

                  /* Pobierz nastepny rekord. */

                  db_row = mysql_fetch_row (result_set);
            }
      }

   g_print("wyjscie z create_find_item_clist...\n");

}

void select_customer(gchar *target_customer_num)
{
   GtkCombo *cbo;
   /* Najpierw wybierzemy odpowiedni element obiektu cbo_customer_number, zgodnie
    * parametrem wejsciowym, target_customer_num, a nastepnie
    * wywolamy fill_customer_info().
    */
   cbo = GTK_COMBO(lookup_widget(frm_main, "cbo_customer_number"));
   gtk_entry_set_text(GTK_ENTRY(cbo->entry), target_customer_num);


   fill_customer_info();
}

void enter_found_customers(gchar *str)
{
   /* Funkcja szuka pasujacych elementow do lancucha str,
    * ktory jest parametrem wywolania funkcji, a nastepnie wprowadza
    * pasujace rekordy do clist_found_customer.
    * Najpierw trzeba polaczyc sie z baza danych i pobrac  
    * mozliwe (pasujace) rekordy
    *
    * Bedziemy przeszukiwac tylko te pola tabeli,
    * ktore zawieraja lancuchy znakowe.
    */ 

   MYSQL_RES   *result_set;
   MYSQL_ROW   db_row;
   gchar       *sql;
   gchar       *clist_row[17] = {"","","","","",
                                 "","","","","",
                                 "","","","","",
                                 "",""};
   gint        counter;
   gint        number_of_rows;

   g_print("starting enter_found_customer...\n");
  
   get_conx();

   /* W MySQL znak procenta jest znakiem zastepczym 
    * pasujacych wynikow.
    */

   sql = g_strconcat("select * from tbl_klienci where nazwa like '%",
                     str,
                     "%' or dost_adr1 like '%",
                     str,
                     "%' or dost_adr2 like '%",
                     str,
                     "%' or dost_miasto like '%",
                     str,
                     "%' or dost_stan like '%",
                     str,
                     "%' or dost_kod like '%",
                     str,
                     "%' or rach_adr1 like '%",
                     str,
                     "%' or rach_adr2 like '%",
                     str,
                     "%' or rach_miasto like '%",
                     str,
                     "%' or rach_stan like '%",
                     str,
                     "%' or rach_kod like '%", 
                     str,
                     "%' or kontakt_imie like '%", 
                     str,
                     "%' or kontakt_nazw like '%", 
                     str,
                     "%' or tel like '%", 
                     str,
                     "%' or stanowisko like '%", 
                     str,
                     "%' or uwagi like '%", 
                     str,
                     "%'", 
                     0L);
  
   g_print("sql jest :  %s\n", sql);

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Proba odnalezienia danych za pomoca mysql_query nie powiodla sie...\n");
      }
   else
      {
          result_set = mysql_store_result (conx);
          db_row = mysql_fetch_row (result_set);

          /* Wyczyszczenie wszystkich elementow listy. */
          gtk_clist_clear(GTK_CLIST(lookup_widget(frm_find_customer, 
                                                  "clist_found_customer")));

          number_of_rows = mysql_num_rows(result_set);
          g_print("liczba rekordow:  %i", number_of_rows);
 
          for (counter = 0; counter < number_of_rows; counter++)
            { 
                  clist_row[0] = db_row[0];
                  clist_row[1] = db_row[1];
                  clist_row[2] = db_row[2];
                  clist_row[3] = db_row[3];
                  clist_row[4] = db_row[4];
                  clist_row[5] = db_row[5];
                  clist_row[6] = db_row[6];
                  clist_row[7] = db_row[7];
                  clist_row[8] = db_row[8];
                  clist_row[9] = db_row[9];
                  clist_row[10] = db_row[10];
                  clist_row[11] = db_row[11];
                  clist_row[12] = db_row[12];
                  clist_row[13] = db_row[13];
                  clist_row[14] = db_row[14];
                  clist_row[15] = db_row[15];
                  clist_row[16] = db_row[16];
                  clist_row[17] = db_row[17];
   /* Dolaczenie rekordu do clist_found_customer. */
                  gtk_clist_append(GTK_CLIST(lookup_widget(frm_find_customer, 
                                  "clist_found_customer")),
                                   clist_row);

                  /* Pobierz nastepny rekord. */

                  db_row = mysql_fetch_row (result_set);
            }
      }
}
void write_order()
{
   /* Funkcja tworzy plik om odpowiedniej nazwie, pobiera
    * dane z roznych okien aplikacji, a nastepnie zapisuje
    * zamowienie.  Domyslnie korzysta z katalogu biezacego.
    */

   gchar      *str_now;
   time_t     now;
   GtkCList   *target_clist;
   gint       number_of_line_items;
   GtkCombo   *cbo;
   gchar      *cust_name_num;
   gchar      *file_name;
   FILE       *fp;
   gchar      *str_ship_to_csz, *str_bill_to_csz;
   gint       counter;
   gchar      *cell_item_number, *cell_description, *cell_quantity, *cell_price;
   gchar      *str_order_total, *str_order_total_formatted;

   /* Najpierw sprawdzimy, czy nie popelniono podstawowych bledow.
    * Czy wybrano klienta?
    */

   cbo = GTK_COMBO(lookup_widget(frm_main, "cbo_customer_number"));

   if (g_strcasecmp(gtk_entry_get_text (GTK_ENTRY(cbo->entry)), "New") == 0)
       {
            g_print("Nowy rekord klienta, nie wazny do utworzenia zamowienia...\n");
            
            return;
       }

   if (g_strcasecmp(gtk_entry_get_text (GTK_ENTRY(cbo->entry)), "") == 0)
       {
            g_print("Brak rekordu klienta, nie mozna utworzyc zamowienia...\n");
            
            return;
       }

   /* Czy zamowiono jakiekolwiek towary? */

   target_clist = GTK_CLIST(lookup_widget(frm_items_ordered, 
                            "clist_items_ordered"));
   number_of_line_items = ((target_clist)->rows);

   if (number_of_line_items == 0)
     {
        
        
        return;
     }

   /* Po sprawdzeniu, czy nie ma ewentualnych bledow, wygenerujemy
    * plik zamowienia.
    *
    * Najpierw nazwa uzytkownika i jego numer.
    */

   cust_name_num = g_strconcat(gtk_entry_get_text(
                                 GTK_ENTRY(lookup_widget(frm_main, 
                                           "entry_customer_name"))),
                               ".",
                               gtk_entry_get_text(GTK_ENTRY(cbo->entry)),
                               ".",
                               0L 
                               ); 
 
   g_print("cust_name_num: %s\n", cust_name_num);
   
   /* Nastepnie, data i dokladny . czas zlozenia zamowienia
    *
    * Trzeba wywolac funkcje g_strstrip(), bo funkcja ctime() zwraca
    * znak CR/LF na koncu lancucha.  g_strstrip()
    * usuwa wszystkie znaki ktorych nie nalezy drukowac z poczatku i konca
    * lancucha, ktory jest parametrem wywolania.
    */

   time(&now);
   str_now = g_strstrip((gchar *) ctime(&now));

   g_print("ctime returns:  %s\n", str_now);

   /* Teraz polaczymy wszystkie dane tak, aby powstala nazwa pliku.
    * Wszystkie spacje i dwukropki zostana zastapione kropkami.
    */

   file_name = g_strconcat(cust_name_num, str_now, ".txt", 0L);
   g_print("Nazwa pliku to %s\n", file_name);

   /* Funkcja g_strdelimit() zastepuje wszystkie znaki niepozadane
    * znakami podanymi jako trzeci parametr.  W tym przypadku
    * kazda spacja, dwukropek lub przecinek zostanie zastapiony
    * kropka.
    *
    * Zwrocmy uwage na to, ze drugi parametr funkcji podano w cudzyslowiu, 
    * a trzeci - w apostrofach.
    *
    * Chodzi o to, aby utworzyc nazwe pliku bedaca opisem jego zawartosci 
    * ale nie powodujaca problemu w wiekszosci systemow operacyjnych.
    * Pamietajmy, ze wynik dzialania naszego programu zostanie
    * przekazany do innego komputera. Najprawdopodobniej skorzystamy
    * tu z FTP i przeslemy pliki zamowien do komputera 
    * z systemem Windows.
    */

   file_name = g_strdelimit(file_name, " :,", '.');

   g_print("file_name is now %s\n", file_name);

   /* Otwarcie pliku do zapisu. */

   if ((fp = fopen (file_name, "w")) == 0L)
      {
          /* Nie mozna zapisac do pliku. */
          
      }
   else
      {
         /* Plik otwarty poprawnie i gotowy do zapisu. */
         

         /* Sformatujemy odpowiednio plik. Zalozymy, ze w jednym wierszu,
          * zmiesci sie 70 znakow, choc wlasciwie moglibysmy przyjac 80
          * 
          */

         /* Najpierw umiescimy naglowek:  data, czas, nazwa klienta
          * i jego numer.
          */

         fprintf(fp, "Specialty Electrical Supply, Inc."); fprintf(fp, "\n");
         fprintf(fp, "Formularz zamowienia");               fprintf(fp, "\n");
         fprintf(fp, "================================="); fprintf(fp, "\n");
                                                           fprintf(fp, "\n");

         fprintf(fp, str_now);              fprintf(fp, "\n");
         fprintf(fp, cust_name_num);        fprintf(fp, "\n");
                                            fprintf(fp, "\n");

         /* Zapiszemy odpowiednie adresy. */

         fprintf(fp, right_pad("Dostarczyc pod adres", " ", 35));    
         fprintf(fp, "Rachunek pod adres"); fprintf(fp, "\n");

         fprintf(fp, right_pad(gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_ship_to_addr1"))), " ", 35));
         fprintf(fp, gtk_entry_get_text(GTK_ENTRY(lookup_widget(frm_main, 
                                   "entry_bill_to_addr1"))));
         fprintf(fp, "\n");

         fprintf(fp, right_pad(gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_ship_to_addr2"))), " ", 35));
         fprintf(fp, gtk_entry_get_text(GTK_ENTRY(lookup_widget(frm_main, 
                                   "entry_bill_to_addr2"))));
         fprintf(fp, "\n");

         /* Zebranie wymaganych informacji: miasto, stan, kod pocztowy. */

         str_ship_to_csz = g_strconcat
                           (
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_ship_to_city"))), " ",
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_ship_to_st"))), "  ",
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_ship_to_zip"))),
                               0L
                           );

         str_bill_to_csz = g_strconcat
                           (
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_bill_to_city"))), " ", 
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_bill_to_st"))), "  ", 
                               gtk_entry_get_text(GTK_ENTRY(
                                   lookup_widget(frm_main, 
                                   "entry_bill_to_zip"))),
                               0L
                           );

         fprintf(fp, right_pad(str_ship_to_csz, " ", 35));
         fprintf(fp, str_bill_to_csz);
         fprintf(fp, "\n");
         fprintf(fp, "\n");

         fprintf(fp, "Przedmioty zamowienia\n");
         fprintf(fp, "========================\n");
         fprintf(fp, "\n");

         fprintf(fp, right_pad("Numer towaru", "-", 12));
         fprintf(fp, right_pad("Opis towaru", "-", 37));
         fprintf(fp, left_pad("Liczba sztuk", "-", 8));
         fprintf(fp, left_pad("Cena", "-", 13));
         fprintf(fp, "\n");
         /*Iteracja po elementach clist_items_ordered
          * i pobranie danych o zamowieniu */
         for (counter = 0; counter < number_of_line_items; counter++)
           {
               gtk_clist_get_text(target_clist, counter, 0, &cell_item_number);
               gtk_clist_get_text(target_clist, counter, 1, &cell_description);
               gtk_clist_get_text(target_clist, counter, 2, &cell_quantity);
               gtk_clist_get_text(target_clist, counter, 3, &cell_price);
              
               fprintf(fp, right_pad(cell_item_number, " ", 12));
               fprintf(fp, right_pad(cell_description, " ", 40));
               fprintf(fp, left_pad(cell_quantity, " ", 5));
               fprintf(fp, left_pad(cell_price, " ", 13));

               fprintf(fp, "\n");
    
               str_order_total = g_strdup_printf("%f", dbl_order_total);
               str_order_total_formatted = g_strndup(str_order_total,
                                          strcspn(str_order_total, ".") +3);

           }
         fprintf(fp, left_pad("=============", " ", 70)); fprintf(fp, "\n");
         fprintf(fp, left_pad(str_order_total_formatted, " ", 70)); 
       
         fprintf(fp, "\n");

         fprintf(fp, "Uwagi do zamowienia\n");
         fprintf(fp, "==============\n");

         fprintf(fp, gtk_editable_get_chars(GTK_EDITABLE(lookup_widget(frm_main,
                   "txt_order_comments")), 0, -1));

         fprintf(fp, "\n");

         fclose(fp);

         
      }

}

gchar *right_pad(gchar *in_str, gchar *pad_char, gint final_length)
{
   /* Ta funkcja wypelnia znakami prawa czesc lancucha in_str 
    * (zapewnia wyrownanie do prawej) */
   while (strlen(in_str) < final_length)
     {
        in_str = g_strconcat(in_str, pad_char, 0L);
     }

   return in_str;
}

gchar *left_pad(gchar *in_str, gchar *pad_char, gint final_length)
{
   /* Ta funkcja wypelnia znakami lewa czesc lancucha in_str 
    * (zapewnia wyrownanie do lewej) */

   while (strlen(in_str) < final_length)
     {
        in_str = g_strconcat(pad_char, in_str, 0L);
     }

   return in_str;
}
void update_database()
{
   /* Ta procedura bedzie uaktualniac baze danych sesi, jesli uzytkownik
    * dokona zmian w rekordzie klienta.
    */

   gchar    *sql;
   GtkCombo *cbo;

   get_conx();

   cbo = GTK_COMBO(lookup_widget(frm_main, "cbo_customer_number"));

   sql = g_strconcat("update tbl_klienci set ",
                     "nazwa = '", gtk_entry_get_text(
                                   GTK_ENTRY(lookup_widget(frm_main, 
                                   "entry_customer_name"))), "', ", 
                     "dost_adr1 = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_ship_to_addr1"))), "', ", 
                     "dost_adr2 = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_ship_to_addr2"))), "', ", 
                     "dost_miasto = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_ship_to_city"))), "', ", 
                     "dost_stan = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_ship_to_st"))), "', ", 
                     "dost_kod = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_ship_to_zip"))), "', ", 
                     "rach_adr1 = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_bill_to_addr1"))), "', ", 
                     "rach_adr2 = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_bill_to_addr2"))), "', ", 
                     "rach_miasto = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_bill_to_city"))), "', ", 
                     "rach_stan = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_bill_to_st"))), "', ", 
                     "rach_kod = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_bill_to_zip"))), "', ", 
                     "kontakt_imie = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_first"))), "', ", 
                     "kontakt_nazw = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_last"))), "', ", 
                     "tel = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_phone"))), "', ", 
                     "stanowisko = '", gtk_entry_get_text(GTK_ENTRY(
                                               lookup_widget(frm_main, 
                                               "entry_title"))), "', ", 
                     "uwagi = '", gtk_editable_get_chars(GTK_EDITABLE(
                                               lookup_widget(frm_main, 
                                               "txt_customer_comments")), 
                                               0, -1), "' ", 
                     "where numer = ", gtk_entry_get_text(GTK_ENTRY(cbo->entry)),
                     0L
                    );

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Nie mozna uaktualnic rekordu,"
                  " skontaktuj sie z administratorem...\n");
          
      }
   else
      {
  g_print("Rekord klienta uaktualniono...\n");
         

      }

   g_print("sql %s\n", sql);

}

void create_new_customer()
{
   /* Ta funkcja tworzy nowy rekord klienta poprzez wprowadzenie danych do
    * pol numer i nazwa. Dzieki autonumeracji MySQL tworzy nowy numer klienta
    * w polu numer. Nastepnie, funkcja zwraca maksymalna wartosc kolumny
    * numer. 
    *
    * W okbnie frm_main, na podstawie nowego numeru, pokazywane sa
    * poczatkowe dane nowego klienta, ktore nalezy uzupelnic.
    * Uzytkownik wprowadza potrzebne dane - nr telefonu, 
    * adres, i tak dalej.
    */

   gchar       *sql;
   MYSQL_RES   *result_set;
   MYSQL_ROW   db_row;

   get_conx();

   sql = "insert into tbl_klienci (nazwa) values ('nowy klient')";

   /* Przeslanie zapytania do bazy. */

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Utworzenie nowego rekordu nie powiodlo sie...\n");
          return;
      }
   else
      {
          g_print("Utworzono rekord nowego klienta...\n");
          

      }

   /* Odswierzenie listy rozwijanej z numerami klientow. */
 
   gtk_combo_set_popdown_strings(GTK_COMBO(lookup_widget(frm_main, 
                                             "cbo_customer_number")),
                                get_customer_numbers());

   /* Pobranie max(numer) z tbl_klienci, czyli wartosci klucza rekordu 
    * nowego klienta
    */

   sql = "select max(numer) from tbl_klienci";

   if (mysql_query (conx, sql) != 0)
      {
   g_print("Nie mozna odczytac informacji o nowym kliencie...\n");
          return;
      }
   else
      {
          g_print("Utworzono rekord nowego klienta...\n");

          result_set = mysql_store_result (conx);
          db_row = mysql_fetch_row (result_set);
 
          /* Teraz do cbo_customer_number trzeba wprowadzic
           * numer nowego klienta. Pozostale pola (opsrocz nazwa)
           * sa jeszcze puste, trzeba wypelnic je  
           * odpowiednimi danymi.
           */

          select_customer(db_row[0]);

          

      }
}

