use commish;

/* Ten plik laduje wszystkie tabele oprocz tbl_place (ktora
 * zaladowano za pomoca Listing 8.1).
 *
 * zycie:
 *
 * % mysql -t < listing.8.2
 */

/* Ponizsza tabela zawiera wykaz
 * pracownikow, ktorzy moga otrzymac zaplate.
 */

drop table if exists tbl_prac;

create table tbl_prac
(
      nr_wiersza       SMALLINT     PRIMARY KEY    AUTO_INCREMENT,
      sprzedawca       char(4),
      nazwisko         varchar(50),
      imie             varchar(35),
      kraj             varchar(50),
      prowizja       decimal(6,4)
);

/* W ponizszej instrukcji LOAD DATA fragment
 * w nawiasach okraglych to wykaz kolumn.
 * Ten wykaz jest wskazowka dla MySQL, gdzie nalezy
 * umiescic odpowiednie dane. Pole nr_wiersza jest pierwszym
 * polem tabeli i jest pomijane w trakcie ladowania
 * danych.
 */

LOAD DATA INFILE "/home/ch08/tbl_prac.txt"
    INTO TABLE tbl_prac
    FIELDS TERMINATED BY ','
    IGNORE 1 LINES
    (sprzedawca,
     nazwisko,
     imie,
     kraj,
     prowizja);

create index idx_prac_sprzedawca on tbl_prac (sprzedawca);

/* Ponizsza tabela przechowuje stawki dla
 * poszczegolnych krajow.
 */

drop table if exists tbl_stawki;

create table tbl_stawki
(
      nr_wiersza SMALLINT    PRIMARY KEY    AUTO_INCREMENT,
      kraj     varchar(50),
      stawka        decimal(8,2)
);

LOAD DATA INFILE "/home/ch08/tbl_stawki.txt"
    INTO TABLE tbl_stawki
    FIELDS TERMINATED BY ','
    IGNORE 1 LINES
    (kraj,
     stawka);

create index idx_kraj on tbl_stawki (kraj);

/* Ponizsza tabela zawiera liste sprzedawcow, ktorzy
 * dostaja prowizje z faktur wystawionych w okreslonym stanie.
 */

drop table if exists tbl_prac_placa_stan;

create table tbl_prac_placa_stan
(
      nr_wiersza SMALLINT    PRIMARY KEY    AUTO_INCREMENT,
      sprzedawca  varchar(50),
      stan       varchar(20)
);

LOAD DATA INFILE "/home/ch08/tbl_prac_placa_stan.txt"
    INTO TABLE tbl_prac_placa_stan
    FIELDS TERMINATED BY ','
    IGNORE 1 LINES
    (sprzedawca,
     stan);

/* Ponizsza tabela zawiera liste sprzedawcow, ktorzy
 * dostaja prowizje na podstawie kodow pocztowych.
 */

drop table if exists tbl_prac_placa_kod;

create table tbl_prac_placa_kod
(
      nr_wiersza SMALLINT    PRIMARY KEY    AUTO_INCREMENT,
      sprzedawca varchar(50),
      kod_pocztowy     varchar(20)
);

LOAD DATA INFILE "/home/ch08/tbl_prac_placa_kod.txt"
    INTO TABLE tbl_prac_placa_kod
    FIELDS TERMINATED BY ','
    IGNORE 1 LINES
    (sprzedawca,
     kod_pocztowy);



create index idx_kod_kod_pocztowy on tbl_prac_placa_kod (kod_pocztowy);
create index idx_kod_sprzedawca   on tbl_prac_placa_kod (sprzedawca);

/* Pozostaly ostatnie dane do zaladowania
 *
 * Ponizej znajduja sie pozostale tabele.
 *
 * Dla tabel , ktore maja niewielkie rozmiary, albo dla tych, ktore
 * nie maja odpowiednich plikow z danymi, utworzono
 * osobne instrukcje insert.
 *
 * Kolejna tabela zawiera liste pracownikow, ktorzy otrzymaja
 * prowizje od kazdej faktury wystawionej na swiecie.
 */

drop table if exists tbl_prac_placa_swiat;

create table tbl_prac_placa_swiat
(
      nr_wiersza    SMALLINT    PRIMARY KEY    AUTO_INCREMENT,
      sprzedawca    varchar(4)
);

insert into tbl_prac_placa_swiat(sprzedawca)
            values('4838'), ('5383');

/* Kolejna tabela zawiera wykaz pracownikow,
 * oplacanych w danym kraju.
 */

drop table if exists tbl_prac_placa_kraj;

create table tbl_prac_placa_kraj
(
      nr_wiersza    SMALLINT    PRIMARY KEY    AUTO_INCREMENT,
      sprzedawca    varchar(4),
      kraj          varchar(50)
);

insert into tbl_prac_placa_kraj(sprzedawca, kraj)
       values ('4070', 'United States'),
              ('4515', 'United States'),
              ('4191', 'United States'),
              ('5646', 'United States'),
              ('4333', 'China'),
              ('3686', 'Germany'),
              ('5151', 'Poland'),
              ('5919', 'Honduras');

/* Kolejna tabela zawiera liste pracownikow ktorzy
 * dostaja prowizje w zaleznosci od faktur wystawionych przez innych sprzedawcow
 */

drop table if exists tbl_prac_placa_inni;

create table tbl_prac_placa_inni
(
    nr_wiersza    SMALLINT      PRIMARY KEY     AUTO_INCREMENT,
    oplacany_prac varchar(4),
    placa       varchar(4)
);

Insert into tbl_prac_placa_inni (oplacany_prac, placa)
       values ('4070', '4515'),
              ('4070', '4191'),
              ('3686', '5151');

/* Kolejna tabela zawiera prowizje. W kazdym miesiacu
 * ta tabela jest wypelniana nowymi danymi,
 * ta operacja rozpoczyna caly proces
 * przetwarzania danych danego miesiaca.
 *
 * Wypelnienie tej tabeli odpowiednimi danymi bedzie
 * skladac sie z kilku etapow. dane tej tabeli beda potrzebne
 * do tworzenia odpowiednich raportow dla sprzedawcow
 * i dyrektorow regionalnych.
 */

drop table if exists tbl_prowizje;

/* Zauwazmy, ze numer wiersza jest zmienna typu
 * INT a nie SMALLINT.  Jest tak, gdyz tabela bedzie
 * miala wiecej rekordow niz 32767 juz po pierwszym
 * lub drugim miesiacu dzialania programu. W MySQL
 * mozna skorzystac takze z MEDIUMINT, ktorej wartoscia
 * maksymalna jest 8388607, ale INT ma wiekszy zakres -  ponad dwa
 * miliardy, co powinno wystarczyc.
 */

create table tbl_prowizje
(
     nr_wiersza    INT    PRIMARY KEY    AUTO_INCREMENT,
     sprzedawca    varchar(4),
     nr_faktury    varchar(10),
     prowizja      decimal(8,2)
);

create index idx_prowizje_sprzed on tbl_prowizje (sprzedawca);
create index idx_prowizja_fakt   on tbl_prowizje (nr_faktury);

/* Na koniec, pozostaly tabele pomocne w administrowaniu i
 * zabezpieczeniu programu.
 *
 * Utworzymy trzy tabele. Pierwsza bedzie przechowywac dane
 * z uprawnieniami (czyli dane uprawnionych uzytkownikow),
 * w drugiej umiescimy nazwy uzytkownikow i hasla, a w trzeciej
 * bedziemy zapisywac zdarzenia (kto uruchomil program, kiedy).
 */

drop table if exists tbl_uprawnienia;

create table tbl_uprawnienia
(
     nazwa        varchar(35),
     uprawnienie         varchar(35)
);

insert into tbl_uprawnienia
       values ('pracownik', 'przetwarzanie'),
              ('pracownik', 'tbl_stawki'),
              ('pracownik', 'tbl_prac'),
              ('pracownik', 'tbl_prac_placa_kraj'),
              ('pracownik', 'tbl_prac_placa_swiat'),
              ('pracownik', 'tbl_prac_placa_stan'),
              ('pracownik', 'tbl_prac_placa_kod'),
              ('pracownik', 'tbl_place'),
              ('pracownik', 'tbl_prowizje'),
              ('pracownik1', 'tbl_prac_placa_swiat'),
              ('pracownik1', 'tbl_place'),
              ('pracownik1', 'tbl_prowizje'),
              ('szef1', 'tbl_stawki'),
              ('szef1', 'tbl_prac'),
              ('szef1', 'tbl_prac_placa_kraj'),
              ('szef1', 'tbl_prac_placa_swiat'),
              ('szef1', 'tbl_prac_placa_stan'),
              ('szef1', 'tbl_prac_placa_kod'),
              ('szef1', 'tbl_place'),
              ('szef1', 'tbl_prowizje');

drop table if exists tbl_bezpieczenstwo;

/* W ponizszej tabeli znajduja sie
 * nazwy uzytkownika i hasla.
 *
 */

create table tbl_bezpieczenstwo
(
      nazwa_uzytkownika    varchar(35),
      haslo     varchar(35)
);

/* Funkcja password() sprawdza nazwe uzytkownika z haslem
 * Funkcja password() bedzie sczytywac dane wpisane w polu
 * entry_password i porownywac wpisane haslo z istniejacymi
 * w bazie danych.
 */

insert into tbl_bezpieczenstwo
       values ('pracownik',  password('smile')),
              ('pracownik1' ,  password('salsa')),
              ('szef1'    ,  password('sippycup'));


drop table if exists tbl_zdarzenia;

/* W ponizszej instrukcji skorzystano z typu  TIMESTAMP
 * ktory nie tylko tworzy kolumne znacznikow czasu, ale
 * takze domyslnie wpisuje biezaca date i czas systemowy.
 */

create table tbl_zdarzenia
(
      znacznik_czasu      TIMESTAMP   NOT NULL,
      nazwa_uzytkownika      varchar(35),
      zdarzenie           varchar(255)
);

insert into tbl_zdarzenia (nazwa_uzytkownika, zdarzenie)
       values ('administrator', 'Utworzenie i zaladowanie bazy danych');

select * from tbl_zdarzenia;
